#!/bin/sh
#set -x

print_Usage() {
		echo ""
		echo "-------- QLogic Corporation -- Firmware Dump Utility --------"
		echo ""
		echo "./ql_dump.sh -l ethX|ALL"
		echo "Display firmware dump information on a \"single\" or \"all\" qlcnic and nx_nic/netxen_nic adapter"
		echo ""
		echo "./ql_dump.sh -s ethX|ALL"
		echo "Save firmware dump from a \"single\" or \"all\" qlcnic and nx_nic/netxen_nic adapter"
		echo "firmware dump will be saved in directory \"/opt/QLogic_Corporation/FW_Dumps\""
		echo "for qlcnic driver udev script \"/lib/udev/qlcnic_udev.sh\" will save the dump automatically"
		echo "Saved file will be in \"driver-name_fw_dump_ethX_date_time.txt.gz\" format"
		echo ""
		echo "./ql_dump.sh -f ethX|ALL"
		echo "Force firmware dump on a \"single\" or \"all\" qlcnic and nx_nic/netxen_nic adapter"
		echo "While performing force dump please make sure that fw dump is enabled"
		echo ""
		echo "./ql_dump.sh -c ethX|ALL 1|0"
		echo "Enable(1)/Disable(0) firmware dump on a \"single\" or \"all\" qlcnic and nx_nic/netxen_nic adapter"
		echo ""
		echo "./ql_dump.sh -m ethX|ALL <Mask-Value>"
		echo "Change the current firmware dump capture mask on a \"single\" or \"all\" qlcnic and nx_nic/netxen_nic adapter"
		echo "Allowed firmware dump capture mask values are 0x3, 0x7, 0xf, 0x1f, 0x3f, 0x7f, and 0xff"
		echo "Default firmware dump capture mask value is 0x1f"
		echo ""
		echo "where ethX is any interface name belonging to qlcnic and nx_nic/netxen_nic adapter"
		echo ""
		exit 1
}


err() {
                echo "$@" >&2
		logger -t "${0##*/}[$$]" "$@" &>/dev/null
		if [ $? -eq 0 ];then
			return
		else
			echo "$@"
		fi		
}

verify_options_and_no_of_arg(){

	if [ "$option" == "-l" -o "$option" == "-f" -o "$option" == "-s" ];then
        	option_verify=1
        	if [ $NOA -ne 2 ];then
			echo ""
                	echo "Please pass valid number of arguments"
                	echo "Operation aborted"
                	print_Usage
        	fi
	fi

	if [ "$option" == "-c" -o "$option" == "-m" ];then
        	option_verify=1
         	if [ $NOA -ne 3 ];then
			echo ""
                	echo "Please pass valid number of arguments"
                	echo "Operation aborted"
                	print_Usage
        	fi
	fi

	if [ "$option" == "-h" -o "$option" == "--help" ];then
        	print_Usage
        	exit
	fi

	if [ $option_verify -ne 1 ];then
		echo ""
        	echo "Please use valid options"
        	echo "Operation aborted"
        	print_Usage
	fi

}

verify_arg(){

	if [ "$1" == "ALL" ];then
		return 0
	else
		ethtool -i "$1" &>/dev/null
		if [ $? -eq 0 ];then
			ethtool -i "$1" | grep qlcnic &>/dev/null
			if [ $? -eq 0 ];then
				arg_type="qlcnic"
				return 0
			else
				ethtool -i "$1" | grep netxen_nic &>/dev/null
				if [ $? -eq 0 ];then
					arg_type="netxen_nic"
					return 0
				fi
				ethtool -i "$1" | grep nx_nic &>/dev/null
				if [ $? -eq 0 ];then
					arg_type="nx_nic"
					port_no=`ethtool $1 |grep PHYAD |cut -d':' -f2`
					#echo $port_no
					if [ $port_no -ne 0 ] ;then
						echo ""
						echo "nx_nic: Please provide interface belonging to port number 0"
						echo ""
						exit
					fi
					return 0
				fi
			fi
			return 1
		else
			return 1
		fi
				
			
	fi
}

check_drivers(){

	lsmod | grep qlcnic &>/dev/null
	if [ $? -eq 0 ];then
		qlcnic=1
	fi

	lsmod | grep netxen_nic &>/dev/null
	if [ $? -eq 0 ];then
		netxen_nic=1
	fi

	lsmod | grep nx_nic &>/dev/null
	if [ $? -eq 0 ];then
		nx_nic=1
	fi

	if [ $nx_nic -eq 1 -a $netxen_nic -eq 1 ];then
		check_nx_nic=`ls /proc/net/nx_nic/ | wc -l`
		if [ $check_nx_nic -eq 0 ];then
			nx_nic=0
		else
			netxen_nic=0
		fi
	fi

	if [ $qlcnic -eq 1 -o $netxen_nic -eq 1 -o $nx_nic -eq 1 ];then
		return 0
	else
		return 1
	fi
}


verify_driver_version() {

	cat /sys/module/"$1"/version &>/dev/null
	if [ $? -eq 0 ];then
		no_dots=`cat /sys/module/"$1"/version | grep -o "\." | wc -l`
		#echo $no_dots
		no_dots=$((no_dots+1))
		loop_for_dot=1
		while [ $loop_for_dot -le $no_dots ]
		do
			figure=`cat /sys/module/"$1"/version | cut -d'.' -f"$loop_for_dot"`
			if [ $loop_for_dot -eq 1 ];then
				Major=$figure
				#echo $Major
			fi

			if [ $loop_for_dot -eq 2 ];then
				Minor=$figure
				#echo $Minor
			fi

			if [ $loop_for_dot -eq 3 ];then
				Subver=$figure
				#echo $Subver
			fi

			if [ $loop_for_dot -eq 4 ];then
				build_no=$figure
			fi

			loop_for_dot=$((loop_for_dot+1))
		done
		return 0
	else
		return 1
	fi

}


verify_versions(){

	if [ $qlcnic -eq 1 ];then
        	verify_driver_version "qlcnic"
        	if [ $? -eq 0 ];then
                	if [ $Major -gt 5 -o $Minor -gt 0 -o $Subver -gt 25 ];then
                        	qlcnic_version=1
                	fi
        	fi
	fi

	if [ $netxen_nic -eq 1 ];then
        	verify_driver_version "netxen_nic"
        	if [ $? -eq 0 ];then
                	if [ $Major -gt 4 -o $Minor -gt 0 -o $Subver -gt 77 ];then
                        	netxen_version=1
                	fi
        	fi
	fi


        if [ $nx_nic -eq 1 ];then
                verify_driver_version "nx_nic"
                if [ $? -eq 0 ];then
                        if [ $Major -gt 4 -o $Minor -gt 0 -o $Subver -gt 579 ];then
                                nx_nic_version=1
                        fi
                fi
        fi

}

Find_bus_for_eth() {

	if [ "$arg_type" == "qlcnic" -o "$arg_type" == "netxen_nic" -o "$arg_type" == "nx_nic" ];then
		eth_dir=`find /sys/devices/pci* -name "$ETH" | grep "/net/$ETH"`
		if [ "$eth_dir" == "" ] ; then
			eth_dir="find /sys/devices/pci* -name "net:$ETH""
			BUS=`$eth_dir | awk -F/ '{ for (i = NF; i > 0; --i) print $i }' | head -2 | grep "0000" | cut -d'.' -f1 | cut -d':' -f2,3`
			return 0
		else
			eth_dir="find /sys/devices/pci* -name "$ETH""
			BUS=`$eth_dir | awk -F/ '{ for (i = NF; i > 0; --i) print $i }' | head -3 | grep "0000" | cut -d'.' -f1 | cut -d':' -f2,3`
			#echo $BUS ; exit
			return 0
		fi
	fi
}


Find_dev_for_nx_nic_eth() {


	if [ "$1" == "nx_nic" -a $nx_nic -eq 1 ];then

		for i in `ls /proc/net/nx_nic/`
		do
			j=0
			match=0
			cat /proc/net/nx_nic/"$i"/stats &>/dev/null
			if [ $? -eq 0 ];then
				eth_nx=`cat /proc/net/nx_nic/"$i"/stats |grep "Interface Name" |cut -d':' -f2`
				while [ $j -lt ${#netxen_eth[@]} -a $match -eq 0 ]
				do
					if [ "$eth_nx" == " ${netxen_eth[$j]}" ] ;then
						match=1
						#echo "Match found"
			       	 		nx_nic_dev[j]="$i"	
					fi
					j=$((j+1))	
				done
			fi
		done
	fi
}


Find_All(){
                if [ "$1" == "qlcnic" ];then
                        ls /sys/bus/pci/devices/0000\:"$i" | grep net &>/dev/null
                        if [ $? -eq 0 ] ; then
                                ls /sys/bus/pci/devices/0000\:"$i"/net/ &>/dev/null
                                if [ $? -eq 0 ] ; then
                                        qlcnic_eth[index]=`ls /sys/bus/pci/devices/0000\:"$i"/net/`
                                        qlcnic_dev[index]="0000:"$i""
                                        index=$((index+1))
                                else
                                        qlcnic_eth[index]=`ls /sys/bus/pci/devices/0000\:"$i"/ | grep net | cut -d':' -f 2`
                                        qlcnic_dev[index]="0000:"$i""
                                        index=$((index+1))
                                fi
                        fi
                fi

                if [ "$1" == "netxen_nic" -o "$1" == "nx_nic" ];then
                        ls /sys/bus/pci/devices/0000\:"$i" | grep net &>/dev/null
                        if [ $? -eq 0 ] ; then
                                ls /sys/bus/pci/devices/0000\:"$i"/net/ &>/dev/null
                                if [ $? -eq 0 ] ; then
                                        netxen_eth[index]=`ls /sys/bus/pci/devices/0000\:"$i"/net/`
                                        netxen_dev[index]="0000:"$i""
                                        index=$((index+1))
                                else
                                        netxen_eth[index]=`ls /sys/bus/pci/devices/0000\:"$i"/ | grep net | cut -d':' -f 2`
                                        netxen_dev[index]="0000:"$i""
                                        index=$((index+1))
                                fi
                        fi
                fi
}

find_interfaces_single_and_all(){

        if [ "$ETH" == "ALL" ];then
                if [ "$option" == "-f" -o $3 == "nx_nic" ];then
                        index=0
                        for i in `lspci | grep "$1" | grep "$2" | cut -d' ' -f 1 | cut -d'.' -f1 | uniq`
                        do
                            	i="$i".0
                                Find_All "$3"
                        done
                else
                        index=0
                        for i in `lspci | grep "$1" | grep "$2" | cut -d' ' -f 1`
                        do
                                Find_All "$3"
                        done
                fi
        else
		if [ "$option" == "-f" -o $3 == "nx_nic" ];then
			index=0
			for i in `lspci | grep "$1" | grep "$2" | cut -d' ' -f 1 | cut -d'.' -f1 | uniq | grep "$4"`
			do
				i="$i".0
				Find_All "$3"
			done
		else
                	index=0
                	for i in `lspci | grep "$1" | grep "$2" | cut -d' ' -f 1 | grep "$4"`
                	do
                        	Find_All "$3"
                	done
        	fi
	fi
}


find_interfaces(){

                if [ $qlcnic -eq 1 -a $qlcnic_version -eq 1 ];then
                        find_interfaces_single_and_all "QLogic" "Ethernet" "qlcnic" "$BUS"
                fi

                if [ $netxen_nic -eq 1 -a $netxen_version -eq 1 ];then
                        find_interfaces_single_and_all "NetXen" "Ethernet" "netxen_nic" "$BUS"
                fi

		if [ $nx_nic -eq 1 -a $nx_nic_version -eq 1 ];then
			find_interfaces_single_and_all "NetXen" "Ethernet" "nx_nic" "$BUS"
		fi
}


Display_Dump(){

                echo ""
                echo "$3: $1: $2"
                echo "Firmware dump available           :$4"
                echo "Firmware dump size                :$7 bytes"
                echo "Firmware dump enabled             :$5"
                echo "Firmware dump capture mask        :0x$6"
                echo ""
                fw_dump_op=$((fw_dump_op+1))
}

Display_dump_nic(){

        ethtool -i "$1" &>/dev/null
	#echo $1
        if [ $? -eq 0 ];then
                ethtool -i "$1" | grep "$3" &>/dev/null
                if [ $? -eq 0 ];then
			dev_parse=1
                        cat "$4"/fwdump_level &>/dev/null
                        if [ $? -eq 0 ];then
                                PROP1=`cat "$4"/fwdump_level`
                                PROP1=`printf "%x\n" $PROP1`
                                PROP2=`cat "$4"/fwdump_size`
                                ls "$4"/fwdump_state &>/dev/null
                                if [ $? -eq 0 ];then
                                        PROP3=`cat "$4"/fwdump_state`
                                        if [ $PROP3 -ne 0 ] ; then
                                                flag1="Yes"
                                        else
                                                flag1="No"
                                        fi
                                else
                                        flag1="Not Supported"
                                fi
                                if [ $PROP2 -ne 0 ] ; then
                                        flag="Yes"
                                else
                                        flag="No"
                                fi
                                Display_Dump "$1" "$2" "$3" "$flag" "$flag1" "$PROP1" "$PROP2"
                        fi
                fi
        fi
}

Display() {

        if [ "$ETH" == "ALL" ];then
        	Operation_dump "netxen_nic" "Display"
		Operation_dump "nx_nic" "Display"
                Operation_dump "qlcnic" "Display"
	else
		Operation_dump "$arg_type" "Display"
        fi
}


Save_dump_nic(){

        mkdir -p $DFILE_PATH &> /dev/null
        QFWD="$4"/fw_dump
        QFWDS="$4"/fwdump_size
        DFILE=${DFILE_PATH}/"$3"_fw_dump_"$1"_`eval date +%Y%m%d_%H%M%S`.txt
        ethtool -i "$1" &>/dev/null
        if [ $? -eq 0 ];then
                ethtool -i "$1" | grep "$3" &>/dev/null
                if [ $? -eq 0 ];then
			dev_parse=1
			cat "$4"/fwdump_level &>/dev/null
                        if [ $? -ne 0 ];then
                                return
                        else
                                size=`cat ${QFWDS}`
                                if [ "$size" -ne 0 ] ; then
                                        while [ $size -ne 0 ]
                                        do
                                                cat ${QFWD} >> ${DFILE} 2>&1
                                                size=`cat ${QFWDS}`
                                        done

                                        if ! test -s "${DFILE}" ; then
                                                rm ${DFILE}
                                                return
                                        else
						gzip ${DFILE}
                                                err "$3-$2-$1: Firmware dump saved to file \"${DFILE}.gz\" "
                                                fw_dump_op=$((fw_dump_op+1))
                                                return
                                        fi
                                else
                                        no_fw_dump=$((no_fw_dump+1))
                                        return
                                fi
                        fi
                fi
        fi
}

Operation() {

        if [ "$ETH" == "ALL" ];then
                Operation_dump "qlcnic" "$1"
                Operation_dump "netxen_nic" "$1"
                Operation_dump "nx_nic" "$1"
        else
                Operation_dump "$arg_type" "$1"
        fi
}


Operation_dump() {

        index=0
        if [ "$1" == "netxen_nic" -a $netxen_version -eq 1 -a $netxen_nic -eq 1 ];then
                while [ $index -lt ${#netxen_eth[@]} ]
                do
                        "$2"_dump_nic ${netxen_eth[$index]} ${netxen_dev[$index]} "netxen_nic" "/sys/class/net/"${netxen_eth[$index]}"/device/"
                        index=$((index+1))
                done
        fi

        if [ "$1" == "qlcnic" -a $qlcnic_version -eq 1 -a $qlcnic -eq 1 ];then
                while [ $index -lt ${#qlcnic_eth[@]} ]
                do
                       	"$2"_dump_nic ${qlcnic_eth[$index]} ${qlcnic_dev[$index]} "qlcnic" "/sys/class/net/"${qlcnic_eth[$index]}"/device/"
                       	index=$((index+1))
                done
        fi

        if [ "$1" == "nx_nic" -a $nx_nic_version -eq 1 -a $nx_nic -eq 1 ];then
                while [ $index -lt ${#nx_nic_dev[@]} ]
                do
                        "$2"_dump_nic ${netxen_eth[$index]} ${netxen_dev[$index]} "nx_nic" "/proc/net/nx_nic/"${nx_nic_dev[$index]}"/"
                        index=$((index+1))
                done
        fi	
}


Save() {

        ls /lib/udev/qlcnic_udev.sh &>/dev/null
        if [ $? -eq 0 ];then
        	ls /etc/udev/rules.d/99-qlcnic.rules &>/dev/null
        	if [ $? -eq 0 ];then
			qlcnic_udev=1
			qlcnic=0
			no_fw_dump=$((no_fw_dump+1))
        	fi
	fi

	Operation "Save"

        if [ $no_fw_dump -ne 0 -a $fw_dump_op -eq 0 ];then
                echo "Firmware dump is not available"
                echo ""
                exit
	fi
}

CheckDrvrs_VerifyArgs_and_version() {

	check_drivers

	if [ $? -ne 0 ];then
        	echo "No drivers found"
        	echo "Please load the supported drivers (qlcnic/netxen_nic/nx_nic)"
        	exit
	fi

	verify_arg $ETH

	if [ $? -ne 0 ];then
        	echo "Invalid argument \"$ETH\""
        	echo "Make sure \"$ETH\" is valid and available driver interface"
        	exit
	fi

	verify_versions
}


Force_dump_nic() {

        ethtool -i "$1" &>/dev/null
        if [ $? -eq 0 ];then
                ethtool -i "$1" | grep "$3" &>/dev/null
                if [ $? -eq 0 ];then
			dev_parse=1
			cat "$4"/fwdump_level &>/dev/null
			if [ $? -eq 0 ];then
				ls "$4"/fwdump_size &>/dev/null
				if [ $? -eq 0 ];then
				size=`cat "$4"/fwdump_size`
				if [ "$size" -ne 0 ] ; then
					echo ""
					echo "$3-$2-$1: Please save the firmware dump first"
					echo "Force firmware dump can't be performed"
					fw_dump_op=$(( $fw_dump_op + 1 ))
					return
				else
					ls "$4"/fwdump_state &>/dev/null
					if [ $? -eq 0 ];then   
						DS=`cat "$4"/fwdump_state`
						if [ $DS -ne 1 ] ; then
							echo ""
							echo "$3-$2-$1: Firmware dump is not enabled"
							echo "Force firmware dump can't be performed"
							fw_dump_op=$(( $fw_dump_op + 1 ))
							return
						else
							echo ""
							echo 0xdeadfeed > "$4"/fw_dump 2>/dev/null
							if [ $? -eq 0 ];then
								echo "$3-$2-$1: Force firmware dump performed"
								fw_dump_op=$(( $fw_dump_op + 1 ))
								return
							fi
						fi
					else
						echo ""
						echo 0xdeadfeed > "$4"/fw_dump 2>/dev/null
						if [ $? -eq 0 ];then
							echo "$3-$2-$1: Force firmware dump performed"
							fw_dump_op=$(( $fw_dump_op + 1 ))
							return
						fi
					fi
				fi
				fi
			fi
		fi
	fi
}


Enable_dump_nic() {

	ethtool -i "$1" &>/dev/null
	if [ $? -eq 0 ];then
		ethtool -i "$1" | grep "$3" &>/dev/null
		if [ $? -eq 0 ];then
			dev_parse=1
			cat "$4"/fwdump_level &>/dev/null
			if [ $? -eq 0 ];then
				ls "$4"/fwdump_size &>/dev/null
				if [ $? -eq 0 ];then
					echo 0xaddfeed > "$4"/fw_dump 2>/dev/null
					if [ $? -eq 0 ];then
						fw_dump_op=$(( $fw_dump_op + 1 ))
						return
					fi
				fi
			fi
		fi
	fi
}

Disable_dump_nic() {

        ethtool -i "$1" &>/dev/null
        if [ $? -eq 0 ];then
                ethtool -i "$1" | grep "$3" &>/dev/null
                if [ $? -eq 0 ];then
			dev_parse=1
                     	cat "$4"/fwdump_level &>/dev/null
                        if [ $? -eq 0 ];then
				ls "$4"/fwdump_size &>/dev/null
				if [ $? -eq 0 ];then
                                	echo 0xbadfeed > "$4"/fw_dump 2>/dev/null
					if [ $? -eq 0 ];then
                                		fw_dump_op=$(( $fw_dump_op + 1 ))
                                		return
					fi
				fi
                        fi
                fi
        fi
}

verify_mask() {

	hex_prefix=`echo "$1" | cut -c 1-2`
	if [ "$hex_prefix" == "0x" ];then
		printf "%d\n" $1 &>/dev/null
		if [ $? -eq 0 ];then
			mask=`printf "%d\n" $1` &>/dev/null
			case $mask in
		 	3) ;;
			7) ;;
       	       	       15) ;;
               	       31) ;;
               	       63) ;;
                      127) ;;
                      255) ;;
	                *) echo "Invalid capture mask "\"$MASK"\""
	                   echo "Operation aborted"
	                   echo ""
	                   exit;;
	                esac
		else
			echo "\"$1\" is not a valid hex number"
			echo ""
			exit
		fi
	else
		echo "Please provide mask values in hexa-decimal format number"
		echo "for example: \"0xValue\""
		echo ""
		exit
	fi
}


ChangeMask_dump_nic() {

        ethtool -i "$1" &>/dev/null
        if [ $? -eq 0 ];then
                ethtool -i "$1" | grep "$3" &>/dev/null
                if [ $? -eq 0 ];then
			dev_parse=1
                        cat "$4"/fwdump_level &>/dev/null
                        if [ $? -eq 0 ];then
                                echo ${MASK} > "$4"/fwdump_level 2>/dev/null
				if [ $? -eq 0 ];then
                                	fw_dump_op=$(( $fw_dump_op + 1 ))
                                	return
				fi
                        fi
                fi
        fi

}

option=$1
ETH=$2
MASK=$3
DFILE_PATH=/opt/QLogic_Corporation/FW_Dumps
NOA=$#
option_verify=0
netxen_nic=0
qlcnic=0
nx_nic=0
netxen_version=0
qlcnic_version=0
nx_nic_version=0
arg_type=""
BUS=""
fw_dump_op=0
qlcnic_udev=0
no_fw_dump=0
dev_parse=0
build_no=""

# Declares An Array in which All the interface Names and device names will be stored
declare -a netxen_eth
declare -a netxen_dev
declare -a qlcnic_eth
declare -a qlcnic_dev
declare -a nx_nic_dev

if [ "$#" == 0 ] ; then
	print_Usage
fi

verify_options_and_no_of_arg
CheckDrvrs_VerifyArgs_and_version

if [ "$ETH" != "ALL" ];then
	Find_bus_for_eth
fi

find_interfaces
Find_dev_for_nx_nic_eth "nx_nic"


case "$option" in
-l)
	echo ""
	echo "<----- Displaying firmware dump information ----->"
	echo ""
	Operation "Display"
	echo ""
	;;
-s)
	echo ""
	echo "<----- Saving the firmware dump ----->"
	echo ""
	Save
	echo ""
	;;
-f)
	echo ""
	echo "<----- Forcing the firmware dump ----->"
	echo ""
	Operation "Force"
	echo ""
	;;
-c)
	echo ""
	echo "<----- Changing the firmware dump capability ----->"
	if [ "$3" == "0" -o "$3" == "1" ] ; then
		if [ $3 -ne 0 ] ; then
			echo ""
			Operation "Enable"
		else
			echo ""
			Operation "Disable"
		fi
	else
		echo ""
		echo "Please check the argument \"$3\""
		echo "Operation aborted"
		print_Usage
	fi

	if [ $fw_dump_op -ne 0 ] ;then
		if [ $3 -ne 0 ];then
			echo "Firmware dump is enabled"
			echo ""
			exit
		else
			echo "Firmware dump is disabled"
			echo ""
			exit
		fi
	fi
	;;
-m)
	echo ""
	echo "<----- Changing the firmware dump capture mask value ----->"
	echo ""
	verify_mask $MASK
	Operation "ChangeMask"

	if [ $fw_dump_op -ne 0 ] ;then
		echo "Firmware dump mask is changed to \"${MASK}\""
		echo ""
		exit
	fi
	;;
*)
	print_Usage
	;;
esac

if [ $fw_dump_op -eq 0 ];then
       	if [ $qlcnic_version -eq 0 -a $netxen_version -eq 0 -a $nx_nic_version -eq 0 ];then
               	echo ""
		echo "\"qlcnic\" driver requires driver version 5.0.26 or higher"
		echo "\"netxen_nic\" driver requires driver version 4.0.78 or higher"
		echo "\"nx_nic\" driver requires driver version 4.0.580 or higher"
		echo "Operations are not supported for current driver versions"
               	echo ""
               	exit
	else
		if [ $dev_parse -eq 0 ];then
			echo ""
			echo "Drivers interfaces are temporarily unavailable"
			echo ""
			exit
		else
			echo ""
			echo "firmware dump interfaces are not available"
			echo ""
			exit
		fi
       	fi
fi
